/**
 * @file Fonctions d'authentification
 * @author akka vodol
 */

import knex from '../../../db/knex_router';
import * as connectors from './connectors.js';
import * as selectors from './selectors';
import * as list_selectors from './list_selectors';
import { UtilisateurAnonyme, UtilisateurConnecte, 
    AdministrateurConnecte, SuperAdministrateurConnecte } from '../../ldap/ldap_data';


function User(user, ldap_access){
    this.anonymous = Boolean(user.anonymous),
    this.uid = user.uid,
    this.password = user.password,
    this.ldap_access = ldap_access;
}

/**
 * @summary Authentifie un utilisateur anonyme
 * @arg {Object} user - Représente l'utilisateur qui a effectué la requête.  
 * @return {Promise(Boolean)} L'utilisateur possède-t-il les droits indiqués ici ?
 * @rights user
 */
export function anonymous(user){
    return new User(user, new UtilisateurAnonyme());
}

/**
 * @summary Authentifie un utilisateur comme viewer(groupUID)
 * @arg {Object} user - Représente l'utilisateur qui a effectué la requête.  
 * @return {Promise(Boolean)} L'utilisateur possède-t-il les droits indiqués ici ?
 * @rights user
 */
export function loggedIn(user, groupUID){
    return new User(user, new UtilisateurConnecte(user));
}

/**
 * @summary Authentifie un utilisateur comme viewer(groupUID)
 * @arg {Object} user - Représente l'utilisateur qui a effectué la requête.  
 * @return {Promise(Boolean)} L'utilisateur possède-t-il les droits indiqués ici ?
 * @rights user
 */
export function viewer(user, groupUID){
    return null;
}

/**
 * @summary Authentifie un utilisateur anonyme
 * @arg {Object} user - Représente l'utilisateur qui a effectué la requête.  
 * @return {Promise(Boolean)} L'utilisateur possède-t-il les droits indiqués ici ?
 * @rights user
 */
export async function member(user, groupUID){
    let group_list = await selectors.groupsWithMember(user);
    let test = await knex.with('group_list', group_list).select().from('group_list').where('uid', groupUID);
    if(test[0])
        return new User(user, new UtilisateurConnecte(user));
    return await admin(user, groupUID);
}

/**
 * @summary Authentifie un utilisateur comme viewer(groupUID)
 * @arg {Object} user - Représente l'utilisateur qui a effectué la requête.  
 * @return {Promise(Boolean)} L'utilisateur possède-t-il les droits indiqués ici ?
 * @rights user
 */
export async function speaker(user, groupUID){
    return await admin(user, groupUID);
}

/**
 * @summary Authentifie un utilisateur anonyme
 * @arg {Object} user - Représente l'utilisateur qui a effectué la requête.  
 * @return {Promise(Boolean)} L'utilisateur possède-t-il les droits indiqués ici ?
 * @rights user
 */
export async function admin(user, groupUID){
    let adminList = await list_selectors.usersWithAdminRights(user, groupUID);
    return (typeof adminList != "undefined" && adminList.indexOf(user.uid) != -1);
}

/**
 * @summary Authentifie un utilisateur comme viewer(groupUID)
 * @arg {Object} user - Représente l'utilisateur qui a effectué la requête.  
 * @return {Promise(uid)} L'utilisateur possède-t-il les droits de superviseur ? Si oui, renvoie la justification. Sinon, renvoie false.
 * Attention : renvoie true si l'utilisateur a les droits mais sans justification.
 * @rights user
 */
export async function superviser(user, groupUID){
    let supervised_groups = await list_selectors.supervisedGroups(user);
    let i = supervised_groups.indexOf(groupUID);
    if(i < 0)
        return false;
    return supervised_groups.justification(i) || true;
}